def getClient(lang="en", gl="EG", remoteHost="102.190.240.107", osName="Windows", osVersion="10.0", platform="Desktop"):
    client = {
        'hl': lang,
        'gl': gl,
        'remoteHost': remoteHost,
        'deviceMake': '',
        'deviceModel': '',
        'visitorData': 'Cgs5S3R6TGlJaHhYZyierrKhBg%3D%3D',
        'userAgent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/111.0.0.0 Safari/537.36,gzip(gfe)',
        'clientName': 'WEB',
        'clientVersion': '2.20230331.00.00',
        'osName': osName.upper(),
        'osVersion': osVersion,
        'originalUrl': 'https://www.youtube.com/watch?v=hfyLjRZmEFc',
        'platform': platform.upper(),
        'clientFormFactor': 'UNKNOWN_FORM_FACTOR',
        'configInfo': {'appInstallData': 'CJ6usqEGEOWg_hIQzJmvBRCCna8FEOf3rgUQzN-uBRDM9a4FEI73rgUQhYOvBRDks_4SELfgrgUQ4tSuBRDthq8FEL-QrwUQuNSuBRDMrv4SEKLsrgUQieiuBRC4i64FEIvqrgUQ1_-uBRC9tq4FEKC3_hIQ1Lj-Eg%3D%3D'},
        'timeZone': 'Africa/Cairo',
        'browserName': 'Chrome',
        'browserVersion': '111.0.0.0',
        'acceptHeader': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7',
        'deviceExperimentId': 'ChxOekl4T0RNeE1EUTFPRE13TlRNM01qUXpOZz09EJ6usqEGGJ6usqEG',
        'screenWidthPoints': 471,
        'screenHeightPoints': 592,
        'screenPixelDensity': 1,
        'screenDensityFloat': 1,
        'utcOffsetMinutes': 120,
        'userInterfaceTheme': 'USER_INTERFACE_THEME_LIGHT',
        'memoryTotalKbytes': '8000000',
        'clientScreen': 'WATCH',
        'mainAppWebInfo': {
            'graftUrl': '/watch?v=hfyLjRZmEFc',
            'pwaInstallabilityStatus': 'PWA_INSTALLABILITY_STATUS_UNKNOWN',
            'webDisplayMode': 'WEB_DISPLAY_MODE_BROWSER',
            'isWebNativeShareAvailable': True
        }
    }
    return client
